// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package organization

import (
	"context"
	"cp-system-management/errors"
	dbmodel "cp-system-management/model"
	pb_v1 "cp-system-management/pb/v1"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/cache"
	"time"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)


func setTime(p *dbmodel.Organization, req *pb_v1.UpdateOrganizationRequest) {
	if req.Month == 0 {
		return
	}
	now := time.Now()
	switch {
	// 之前没有设置时间，从当前时间开始加
	case req.Month > 0 && p.EndTime == 0:
		p.EndTime = util.AddDate(now, 0, int(req.Month), 0).Unix()
		p.StartTime = now.Unix()
	// 已过期了,从当前时间开始加
	case req.Month > 0 && p.EndTime <= now.Unix():
		p.EndTime = util.AddDate(now, 0, int(req.Month), 0).Unix()
		p.StartTime = now.Unix()
	// 未过期，从计划结束时间开始加
	case req.Month > 0 && p.EndTime > now.Unix():
		p.EndTime = util.AddDate(time.Unix(p.EndTime, 0), 0, int(req.Month), 0).Unix()
	// 未过期，从计划结束时间开始减
	case req.Month < 0 && p.EndTime > now.Unix():
		p.EndTime = util.AddDate(time.Unix(p.EndTime, 0), 0, int(req.Month), 0).Unix()
		if p.EndTime < p.StartTime {
			p.EndTime = p.StartTime
		}

	}

	return
}
// 创建机构
func UpdateOrganization(ctx context.Context, req *pb_v1.UpdateOrganizationRequest) (reply *pb_v1.UpdateOrganizationReply, err error) {
	reply = &pb_v1.UpdateOrganizationReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OrganizationCode == "" {
		return reply, status.Error(10003, "参数错误，机构编码为空")
	}

	if req.Month > 2400 || req.Month < -2400{
		return reply, status.Error(10003, "参数错误，有效期增减范围过大")
	}


	p := &dbmodel.Organization{}
	db := database.DB()

	where := map[string]interface{}{"organization_code": req.OrganizationCode}
	err = p.Find(db, where)
	if err != nil {
		return reply, errors.OrganizationNotExist
	}

	reply.Origin = &pb_v1.UpdateOrganizationRequest{
		OrganizationCode:p.OrganizationCode,
		OrganizationName:p.OrganizationName,
		IsDisable:p.IsDisable,
	}

	// 传入机构名不为空，并且和机构名不一致，查看机构名是否重复
	if req.OrganizationName != "" && req.OrganizationName != p.OrganizationName {
		p1 := &dbmodel.Organization{}
		where := map[string]interface{}{"organization_name": req.OrganizationName}
		err = p1.Find(db, where)
		if err == nil {
			return reply, errors.OrganizationExist
		}
		p.OrganizationName = req.OrganizationName
	}

	p.IsDisable = req.IsDisable
	setTime(p, req)
	if p.EndTime <= p.CreatedAt.Unix() {
		p.IsExpire = true
	}
	if p.IsDisable {
		cache.Redis().Set("organization_active"+p.OrganizationCode, "0")
	} else {
		cache.Redis().Set("organization_active"+p.OrganizationCode, "1")
	}

	fields := map[string]interface{}{
		"organization_name":req.OrganizationName,
		"start_time":p.StartTime,
		"end_time":p.EndTime,
		"is_expire":p.IsExpire,
		"is_disable":p.IsDisable,
	}
	err = p.UpdateSome(database.DB(), fields)
	if err != nil {
		return reply, errors.DataBaseError
	}
	cache.Redis().Set("organization_end_time"+p.OrganizationCode, fmt.Sprintf("%d", p.EndTime))

	return reply, nil
}
